//   C4D3 is free software: you can redistribute it and/or modify it under
// the terms of the GNU Affero General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option) any
// later version.
//type typeOfValue = string | number | boolean | undefined | null;
export class ControlInfo {
    //**********************************************************************
    // Public Class Methods
    //**********************************************************************
    static bind(control, tag, variable) {
        control.describe().getLiveProperty(tag).setVariable(variable);
    }
    static unbind(control, tag) {
        control.describe().getLiveProperty(tag).setVariable(null);
    }
    static transduce(src, dst, srcTag, dstTag) {
        if (src && dst && srcTag && dstTag) {
            var srcProperty = src.describe().getLiveProperty(srcTag);
            var variable = srcProperty.getVariable();
            if (variable == null)
                ControlInfo.unbind(dst, dstTag);
            else
                ControlInfo.bind(dst, dstTag, variable);
        }
        else if ((src && dst && srcTag)) {
            ControlInfo.transduce(src, dst, srcTag, srcTag);
        }
        else if ((src && dst && dstTag)) {
            ControlInfo.transduce(src, dst, dstTag, dstTag);
        }
        else if (src && dst) {
            var tags = src.describe().getTags();
            var i;
            for (i = 0; i < tags.length; i++) {
                ControlInfo.transduce(src, dst, tags[i]);
            }
        }
    }
    //**********************************************************************
    // Constructors and Finalizer
    //**********************************************************************
    constructor(proxy) {
        this.proxy = proxy;
    }
    //**********************************************************************
    // Getters and Setters
    //**********************************************************************
    getControl() {
        return this.proxy.getControl();
    }
    //**********************************************************************
    // Public Methods (Properties)
    //**********************************************************************
    /**
     * Returns a vector of all LiveProperty tags, in the order added.
     */
    getTags() {
        return this.proxy.getTags();
    }
    /**
     * Returns the number of LiveProperty tags.
     */
    getTagCount() {
        return this.proxy.getTagCount();
    }
    /**
     * Returns the LiveProperty tag at the specified index.
     */
    getTagAt(index) {
        return this.proxy.getTagAt(index);
    }
    /**
     * Returns where a tag is in the vector of all tags, or -1 if absent.
     */
    getIndexOfTag(tag) {
        return this.proxy.getIndexOfTag(tag);
    }
    /**
     * Returns the LiveProperty associated with the specified tag, or null if
     * none exists.
     *
     * @param		tag		the name of the LiveProperty to retrieve
     */
    getLiveProperty(tag) {
        return this.proxy.getLiveProperty(tag);
    }
    /**
     * Returns a vector of all LiveProperties, in the order added.
     */
    getLiveProperties() {
        return this.proxy.getLiveProperties();
    }
    /**
     * Returns the property at the specified index.
     */
    getPropertyAt(index) {
        return this.proxy.getPropertyAt(index);
    }
    /**
     * Convenience method to register a listener with all current properties.
     * This method does NOT automatically handle addition and removal of
     * properties to the control. (Use ControlListener to detect such changes.)
     */
    addLivePropertyListener(l) {
        this.proxy.addLivePropertyListener(l);
    }
    /**
     * Convenience method to deregister a listener with all current properties.
     * This method does NOT automatically handle addition and removal of
     * properties to the control. (Use ControlListener to detect such changes.)
     */
    removeLivePropertyListener(l) {
        this.proxy.removeLivePropertyListener(l);
    }
    //**********************************************************************
    // Public Methods (Properties, Variable Access)
    //**********************************************************************
    getVariable(tag) {
        return this.proxy.getVariable(tag);
    }
    setVariable(tag, variable) {
        this.proxy.setVariable(tag, variable);
    }
    getName(tag) {
        return this.proxy.getName(tag);
    }
    setName(tag, name) {
        this.proxy.setName(tag, name);
    }
    getValue(tag) {
        return this.proxy.getValue(tag);
    }
    setValue(tag, object) {
        this.proxy.setValue(tag, object);
    }
    //**********************************************************************
    // Public Methods (Properties, Locking)
    //**********************************************************************
    isLocked() {
        return this.proxy.isLocked();
    }
    //**********************************************************************
    // Public Methods (Events)
    //**********************************************************************
    /**
     * Registers the specified ControlListener to receive ControlEvents
     * whenever a LiveProperty is added to or removed from this ControlInfo.
     */
    addControlListener(l) {
        this.proxy.addControlListener(l);
    }
    /**
     * Unregisters the specified ControlListener.
     */
    removeControlListener(l) {
        this.proxy.removeControlListener(l);
    }
}
//******************************************************************************
//# sourceMappingURL=ControlInfo.js.map