//   C4D3 is free software: you can redistribute it and/or modify it under
// the terms of the GNU Affero General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option) any
// later version.
import { VariableEvent } from "./VariableEvent.js";
import { SharedFlag } from "./SharedFlag.js";
import { LiveProperty } from "./LiveProperty.js";
//import { typeOfValue } from "../index.js";
import { printClass } from "./printClass.js";
export class Variable {
    //**********************************************************************
    // Constructors and Finalizer
    //**********************************************************************
    constructor(prototype, name, _value) {
        this.prototype = prototype;
        this.fflag = new SharedFlag();
        this.eflag = new SharedFlag();
        this.lflag = new SharedFlag();
        this.tflag = new SharedFlag();
        this.sflag = new SharedFlag();
        this.listeners = new Array;
        this.name = name;
        this.value = _value;
        //console.log("Second : Variable " +name+ " created");
        if (printClass.printStatus)
            console.log(this.prototype.value);
        if (printClass.printStatus)
            console.log(this.value);
    }
    //**********************************************************************
    //  Class Methods
    //**********************************************************************
    static initStartup() {
        Variable.ASYNC_EVENTS = false;
        Variable.STARTUP = true;
    }
    static termStartup() {
        Variable.ASYNC_EVENTS = true;
        Variable.STARTUP = false;
    }
    //**********************************************************************
    // Getters and Setters
    //**********************************************************************
    getPrototype() {
        return this.prototype;
    }
    getName() {
        return ((name == null) ? this.prototype.getName : name);
    }
    setName(name) {
        if ((name != null) && (name == (this.name)))
            return;
        this.name = name;
        this.fireEvent(new VariableEvent(this, VariableEvent.NAME_CHANGED, name));
    }
    getValue() {
        return ((this.value == null) ? Variable.prototype.getValue() : this.value);
    }
    setValue(value) {
        //if(printClass.printStatus) 
        //console.log("SetValue in Variable");
        if (this.prototype.getConstant() || this.isLocked())
            return;
        if (printClass.printStatus)
            console.log(this.prototype.classType);
        if (printClass.printStatus)
            console.log(value.__proto__);
        if (printClass.printStatus)
            console.log(value);
        //console.log("check")
        if ((value != null) && !(this.prototype.classType == value.__proto__))
            throw new Error(value.__proto__.name);
        //	 console.log("single check")
        if ((value == null) && (this.value == null))
            return;
        this.value = value;
        //console.log(value==(this.value));
        this.prototype.value = value; //This Step Solved a Major Problem
        var varEvent = new VariableEvent(this, VariableEvent.VALUE_CHANGED, "dummy", value);
        if (printClass.printStatus)
            console.log(varEvent);
        if (printClass.printStatus)
            console.log(this);
        this.fireEvent(varEvent);
    }
    //**********************************************************************
    //  public Methods
    //**********************************************************************
    getType() {
        return Variable.prototype.getType();
    }
    isAssignableFrom(variable) {
        return this.prototype.isAssignableFrom(variable.getPrototype());
    }
    touch() {
        this.fireEvent(new VariableEvent(this, VariableEvent.VALUE_CHANGED));
    }
    getBoundProperties() {
        var properties = new Array();
        var n = this.listeners.length;
        var i = 0;
        for (i = 0; i < n; i++)
            if ((this.listeners[i] instanceof LiveProperty) &&
                (this.listeners[i]).getVariable() == this)
                properties.push(this.listeners[i]);
        return properties;
    }
    //**********************************************************************
    //  Methods (Focusing)
    //**********************************************************************
    isFocusing() {
        //return fflag.getState();
        return (this.isLocked() ? false : this.fflag.getState());
    }
    setFocusing(b, object) {
        if (this.fflag.setState(b, object))
            this.fireEvent(new VariableEvent(this, VariableEvent.FOCUS_CHANGED));
    }
    isSelfFocusing() {
        return this.fflag.getState(Variable);
    }
    setSelfFocusing(b) {
        this.setFocusing(b, Variable);
    }
    //**********************************************************************
    //  Methods (Editing)
    //**********************************************************************
    isEditing() {
        //return eflag.getState();
        return (this.isLocked() ? false : this.eflag.getState());
    }
    setEditing(b, object) {
        if (this.eflag.setState(b, object))
            this.fireEvent(new VariableEvent(this, VariableEvent.EDIT_CHANGED));
    }
    isSelfEditing() {
        return this.eflag.getState(Variable);
    }
    setSelfEditing(b) {
        this.setEditing(b, Variable);
    }
    //**********************************************************************
    //  Methods (Locking)
    //**********************************************************************
    isLocked() {
        return this.lflag.getState();
    }
    setLocked(b, object) {
        if (this.lflag.setState(b, object))
            this.fireEvent(new VariableEvent(this, VariableEvent.LOCK_CHANGED));
    }
    isSelfLocked() {
        return this.lflag.getState(Variable);
    }
    setSelfLocked(b) {
        this.setLocked(b, Variable);
    }
    //**********************************************************************
    //  Methods (Tracking)
    //**********************************************************************
    isTracked() {
        return this.tflag.getState();
    }
    setTracked(b, object) {
        if (this.tflag.setState(b, object))
            this.fireEvent(new VariableEvent(this, VariableEvent.TRACK_CHANGED));
    }
    isSelfTracked() {
        return this.tflag.getState(Variable);
    }
    setSelfTracked(b) {
        this.setTracked(b, Variable);
    }
    //**********************************************************************
    //  Methods (Shared)
    //**********************************************************************
    isShared() {
        return this.sflag.getState();
    }
    setShared(b, object) {
        if (this.sflag.setState(b, object))
            this.fireEvent(new VariableEvent(this, VariableEvent.SHARE_CHANGED));
    }
    isSelfShared() {
        return this.sflag.getState(Variable);
    }
    setSelfShared(b) {
        this.setShared(b, Variable);
    }
    //**********************************************************************
    //  Methods (Events)
    //**********************************************************************
    addVariableListener(l) {
        this.listeners.push(l);
        if (printClass.printStatus)
            console.log("addVariableListener of Variable");
        if (printClass.printStatus)
            console.log(l);
    }
    removeVariableListener(l) {
        this.listeners.splice(this.listeners.indexOf(l), 1);
    }
    //**********************************************************************
    // Override Methods (Disposable)
    //**********************************************************************
    dispose() {
        //if (this.value instanceof Value)
        //	(this.value).disassociate(this);
        this.fireEvent(new VariableEvent(this, VariableEvent.DISPOSING));
    }
    //**********************************************************************
    // Private Methods (Events)
    //**********************************************************************
    fireEvent(e) {
        this.fireVariableChanged(e);
    }
    fireVariableChanged(e) {
        //if(printClass.printStatus) console.log(this.listeners)
        var n = e.getVariable().listeners.length;
        if (printClass.printStatus)
            console.log("Number of Listeners " + n);
        var i = 0;
        for (i = 0; i < n; i++) {
            if (printClass.printStatus)
                console.log("Listener " + i + " : " + this.listeners[i].getControl().ControlName);
        }
        for (i = 0; i < n; i++) {
            if (printClass.printStatus)
                console.log("----------------------- Listener :" + i + "-------------------------------------");
            try {
                if (printClass.printStatus)
                    console.log(this.listeners[i]);
                (this.listeners[i]).variableChanged(e);
            }
            catch (error) {
                //if(printClass.printStatus) console.log((VariableListener)listeners.at(i));
            }
        }
    }
}
//**********************************************************************
//  Class Members
//**********************************************************************
/**
 * If true, notify event handlers using Runnable invoked later.
 */
Variable.ASYNC_EVENTS = true;
Variable.STARTUP = false;
//# sourceMappingURL=Variable.js.map