//   C4D3 is free software: you can redistribute it and/or modify it under
// the terms of the GNU Affero General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option) any
// later version.
// This file is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
// details.
// You should have received a copy of the GNU Affero General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
import { Prototype } from "../LP/Prototype.js";
import { ControlProxy } from "../LP/ControlProxy.js";
import { TableClass } from "../view/table.js";
import { printClass } from "../LP/printClass.js";
//type typeOfValue = string | number | boolean | undefined | null;
//******************************************************************************
// class ControlSlider
//******************************************************************************
export class ControlTable {
    //**********************************************************************
    // Constructors and Finalizer
    //**********************************************************************
    constructor(name, data, xPos, yPos, inputWidth, inputHeight) {
        // test(evt: Event): void
        // {
        //     throw new Error("Method not implemented.");
        // }
        //**********************************************************************
        // Public Class Members (Properties)
        //**********************************************************************
        this.CTAG_Height = "Height";
        this.CTAG_Width = "Width";
        this.CTAG_Opacity = "Opacity";
        this.CTAG_Selection = "Selection";
        this.CTAG_Indication = "Indication";
        this.CTAG_Data = "Data";
        this.TYPE_Height = new Prototype(Number.prototype, this.CTAG_Height, 0);
        this.TYPE_Width = new Prototype(Number.prototype, this.CTAG_Width, 0);
        this.TYPE_Opacity = new Prototype(Number.prototype, this.CTAG_Opacity, 0);
        this.TYPE_Selection = new Prototype(Array.prototype, this.CTAG_Selection, [false]);
        this.TYPE_Indication = new Prototype(Array.prototype, this.CTAG_Indication, []);
        this.TYPE_Data = new Prototype(Array.prototype, this.CTAG_Data, []);
        if (printClass.printStatus)
            console.log("I am in constructor of ControlTable");
        //Create Control Proxy
        //if(printClass.printStatus) console.log(this.TYPE_VALUE);
        this.proxy = new ControlProxy(this);
        //create live property
        this.live_property_Height = this.proxy.add(this.CTAG_Height, this.TYPE_Height, true);
        this.live_property_Width = this.proxy.add(this.CTAG_Width, this.TYPE_Width, true);
        this.live_property_Opacity = this.proxy.add(this.CTAG_Opacity, this.TYPE_Opacity, true);
        this.live_property_Selection = this.proxy.add(this.CTAG_Selection, this.TYPE_Selection, true);
        this.live_property_Indication = this.proxy.add(this.CTAG_Indication, this.TYPE_Indication, true);
        this.live_property_Data = this.proxy.add(this.CTAG_Data, this.TYPE_Data, true);
        this.ControlName = name;
        this.table = new TableClass(this, name, data, xPos, yPos, inputWidth, inputHeight);
        if (printClass.printStatus)
            console.log(name);
    }
    //**********************************************************************
    // Public Methods (Properties)
    //**********************************************************************
    getHeight() {
        return (this.proxy.getValue(this.CTAG_Height));
    }
    setHeight(value) {
        if (printClass.printStatus)
            console.log("setHeight method of ControlTable");
        this.proxy.setValue(this.CTAG_Height, value);
        //this.canvas.setValue(value);
    }
    getColor() {
        return (this.proxy.getValue(this.CTAG_Opacity));
    }
    setColor(value) {
        if (printClass.printStatus)
            console.log("setColor method of ControlTable");
        this.proxy.setValue(this.CTAG_Opacity, value);
        //this.canvas.setValue(value);
    }
    getProxy() {
        //if(printClass.printStatus) 
        //console.log("getProxy method of ControlTable" )
        return this.proxy;
    }
    getWidth() {
        return (this.proxy.getValue(this.CTAG_Width));
    }
    setWidth(value) {
        if (printClass.printStatus)
            console.log("setWidth method of ControlTable");
        this.proxy.setValue(this.CTAG_Width, value);
        //this.canvas.setValue(value);
    }
    //**********************************************************************
    // Override Methods (LivePropertyListener)
    //**********************************************************************
    propertyChanged(e) {
        if (printClass.printStatus)
            console.log("propertyChanged method of ControlTable");
        if (printClass.printStatus)
            console.log(e);
        var tag = e.getLiveProperty().getTag();
        if (printClass.printStatus)
            console.log(tag);
        var ve = e.getVariableEvent();
        if (tag == this.CTAG_Height) {
            if (printClass.printStatus)
                console.log("hurray ControlTable height");
            this.table.setHeight(e.getLiveProperty().getVariable().getValue());
        }
        else if (tag == this.CTAG_Width) {
            if (printClass.printStatus)
                console.log("hurray ControlTable Width");
            this.table.setWidth(e.getLiveProperty().getVariable().getValue());
        }
        else if (tag == this.CTAG_Opacity) {
            if (printClass.printStatus)
                console.log("hurray ControlTable opacity");
            this.table.setOpacity(e.getLiveProperty().getVariable().getValue());
        }
        else if (tag == this.CTAG_Selection) {
            if (printClass.printStatus)
                console.log("hurray ControlTable Selection");
            this.table.setSelection(e.getLiveProperty().getVariable().getValue());
        }
        else if (tag == this.CTAG_Indication) {
            if (printClass.printStatus)
                console.log("hurray ControlTable Hover");
            this.table.setIndication(e.getLiveProperty().getVariable().getValue());
        }
        else if (tag == this.CTAG_Data) {
            if (printClass.printStatus)
                console.log("hurray ControlTable Data");
            this.table.setHoverSelection(e.getLiveProperty().getVariable().getValue());
        }
    }
    //**********************************************************************
    // Private Methods (Graphics)
    //**********************************************************************
    describe() {
        throw new Error("Method not implemented.");
    }
    dispose() {
        throw new Error("Method not implemented.");
    }
}
//******************************************************************************
//# sourceMappingURL=ControlTable.js.map